*                                    WARNING

* The Government of Canada (Statistics Canada) is the owner of all intellectual
* property rights (including copyright) in this software.  Subject to the terms below,
* you are granted a non-exclusive and non-transferable licence to use this software.
*
* This software is provided "as-is", and the owner makes no warranty, either express
* or implied, including but not limited to, warranties of merchantability and fitness
* for any particular purpose.  In no event will the owner be liable for any indirect,
* special, consequential or other similar damages.  This agreement will terminate
* automatically without notice to you if you fail to comply with any term of this
* agreement.

*******************************************************************
* Date: April 2002                                                         *
* Modified:  June 2002                                                   *
*******************************************************************.

**********************************************************************************************
***                                                                                                              ***
***                               BOOTVARE_V21_NLSCY.SPS                                 ***
***                                                                                                              ***
*** This program calculates variance estimates using the bootstrap weights      ***
*** for different types of estimators.  Using SPSS macros, this program can     ***
*** calculate variance estimates for totals, ratios and differences between         ***
*** ratios and persentiles.  It can also calculate variance estimates                   ***
***  for the parameters of a regression and a logistic regression.                       ***
***                                                                                                               ***
*** The program is divided in 2 sections. Throughout the program, it is               ***
*** indicated where the user has to make changes.                                          ***
***                                                                                                               ***
*** Section 1: Declaration of the macro variables                                             ***
***            This is where variables that are going to be used throughout             ***
***            the program have to be defined.  Some changes must be made        ***
***            by the user.  This is also where the files containing the                    ***
***            variables to analyze and the bootstrap weights are read in.               ***
***                                                                                                               ***
*** Section 2: Declaration of the macros                                                         ***
***            This is where the user indicates what analysis they wish to              ***
***            perform.                                                                                       ***
***                                                                                                               ***
*** MACROE_V21_NLSCY.SPS  contains different macros that enable variance***
*** estimation.  For standard  use of the variance estimation program,               ***
*** no modification by the user to  MACROE_V21_NLSCY.SPS is necessary.   ***
***                                                                                                               ***
***                                                                                                               ***
*** Before using the program BOOTVARE_V21_NLSCY.SPS, the user has to prepare    ***
*** an analysis file that contains the variables to analyze.                                  ***
***                                                                                                                ***
*** To calculate a total, ratio or difference between ratios, the                            ***
*** estimates are obtained by summing the weights of the records that have       ***
*** the characteristic of interest.  Therefore, a dummy variable must be              ***
*** created for each of the variables to be analyzed.  This variable takes a          ***
*** value of 1 when the record has the characteristic and 0 otherwise.                ***
***                                                                                                               ***
*** NOTE: This file should only contain the records necessary for the                ***
*** analysis (ex: if the analysis is performed on the age group 12 and over,        ***
*** only the records for individuals 12 and over should be included in                 ***
*** the file.)                                                                                                  ***
**********************************************************************************************.


* All temporary files are written on C:\Temp and each file name has "SPSS_" as a
* prefix. 'C:\Temp\SPSS_All_Tots.sav' contains the Totals, Ratios, and Difference of Ratios and Percentiles
* Parameters File and 'C:\Temp\SPSS_All_Regs.sav' contains the Regressions Saved Parameters File.
* You can delete all "SPSS_" files from C:\Temp after execution. You can also move files like the
* two previously mentioned for further use.

* Results can be found at the very end of the output, in the one or two "Text Output". Do not print
* the whole output which can be very large! Use the "Draft Viewer" if you run into memory problems.


*********************************************************************************
***                               SECTION 1                                              ***
*********************************************************************************
***                                                                                              ***
***   This section lets the user specify the different parameters of     ***
***   interest (variable names, directory names, file names, etc.)      ***
***                                                                                              ***
*********************************************************************************.

SET COM NO HEA ON MES ON ERR ON RES ON JOU ON LEN NONE WID 132 PRI NO MPR NO.


DEFINE !Bootvar()

!Let !Tot=!Null
!Let !Reg=!Null
!LET !Classes = ''


************************************************************************************************************
* SPECIFY THE ANALYSIS DATA FILE TO BE USED (created in step 1) (Folder and file) :   *
************************************************************************************************************.

!LET !Mfile='Folder_and_file_name_of_the_analysis_file_(step 1)'          /*  <- (ex: c:\data\analysis.sav). 


***********************************************************************************************
*   SPECIFY THE BOOTSTRAP WEIGHTS FOLDER AND FILE NAMES :           *
*               NB: In NLSCY the weights files are in *.sd2 format                             *
***********************************************************************************************.

!LET !BWsav='FOLDER_AND_FILE_OF_THE_BOOTSTRAP_WEIGHTS.sd2 '


****************************************************************************************************************************************
* SPECIFY, IF DESIRED, THE BREAKDOWN VARIABLE(S) (EG: PROVINCE, SEX, ETC...):                                  *
****************************************************************************************************************************************.

*!LET !Classes = ' breakdown_variable(s)_or_empty_(keep_the_2_single_quotes) '


*************************************************************************************************************************************
* SPECIFY THE NUMBER OF BOOTSTRAP WEIGHTS TO USE                                                                       *
*                  -  FOR TESTING, B MUST BE >= 2                                                                                              *
************************************************************************************************************************************.

!LET !B =3


***************************************************************************************************************************
* SPECIFY THE DIRECTORY AND THE NAME OF THE FILE THAT CONTAINS THE MACROS               *
***************************************************************************************************************************.

INCLUDE FILE='PATH_TO_THE_FILE\Macroe_v21_NLSCY.sps'.      


**********************************************************************************************.
* SPECIFY THE VARIABLE NAME OF THE IDENTIFIER USED TO LINK THE 
* FILE OF ANALYSIS TO THE BOOTSTRAP WEIGHTS FILE
**********************************************************************************************.

!LET !LinkId=childid




************************************************************************************************
***                                      SECTION 2                                                          ***
************************************************************************************************
***                                                                                                                 ***
***          This section lets the user specify the different analyses of interest.        ***
***                                                                                                                 ***
************************************************************************************************.


*****************************************************************************************************
*** Variance estimates are calculated using the SPSS macros defined in the            ***
*** MACROE_V21.SPS program.  These SPSS macros can be submitted in this      ***  
*** program, to suit the user's needs.                                                                     ***
***                                                                                                                       ***
*** To submit a macro, the statement must specify the macro name and parameters  ***
*** to use.  In this program, the parameters indicate which variables will be                ***
*** used.                                                                                                              ***
***                                                                                                                       ***
*** Each macro submission gives an estimate of the variance of only one                  ***
*** parameter. If more than one parameter and variance need to be                           ***
*** calculated, the macro then must be submitted several times.                              ***
***                                                                                                                       ***
*** A statement for each possible type of analysis appears in the program. They        ***
*** are commented out and the user only needs to run those that are desired.            ***
***                                                                                                                       ***
*** Parameters "!B !Classes/" must be supplied exactly AS IS and BEFORE any       ***
*** Named Parameter                                                                                            ***
***                                                                                                                       ***
*****************************************************************************************************.


*****************************************************************************************************.
!Prepare B=!B /Classes=!Classes /mfile=!Mfile /bwsav=!bwsav /keys=!LinkId.
*****************************************************************************************************.


* TO OBTAIN VARIANCE ESTIMATES OF A TOTAL, RUN:
*********************************************************************.

*!Total !B !Classes/ Var = variable.
*!Let !Tot=1.


* TO OBTAIN VARIANCE ESTIMATES OF A RATIO, RUN:
********************************************************************.

*!Ratio !B !Classes/ Varlist =numerator denominator.
*!Let !Tot=1.


* TO OBTAIN VARIANCE ESTIMATES OF A DIFFERENCE OF RATIO, RUN:
******************************************************************************************.

*!Diffrat !B !Classes/ Varlist = numerator1 denominator1 numerator2 denominator2.
*!Let !Tot=1.


* TO OBTAIN VARIANCE ESTIMATES OF PERCENTILES, RUN:
**********************************************************************************************.

*!PCTILE  !B !Classes/  varlist=Variable Percent.
*!Let !Pct=1.


* TO OBTAIN VARIANCE ESTIMATES OF REGRESSION PARAMETERS, RUN:
**********************************************************************************************.

*!Regress  !B !Classes/    Dep =Dependent_Variable  Indep =Independent_Variable(s)
*!Let !Reg=1.


* TO OBTAIN VARIANCE ESTIMATES OF LOGISTIC REGRESSION PARAMETERS, RUN:
**********************************************************************************************************.

*!Log_Reg !B !Classes/ Cri=SAS    Dep =Dependent_Variable  Indep =Independent_Variable(s)
*!Let !Reg=1.

           * NOTE: Default iteration criteria differs between SAS and SPSS. 
           *       Use Cri=SAS to use SAS default values.



*********************************************************************
*        DO NOT MODIFY THE REMAINING LINES             *
*********************************************************************.

SET PRI NO MPR on.

**** Printing of results ****.
!IF (!Tot=1) !THEN
+ !Print_T !Classes.   /* Printing of Totals, Ratios and Differences of Ratios results
!IFEND
!IF (!Reg=1) !THEN
+ !Print_R !Classes.   /* Printing of Linear and Logistic Regressions results
!IFEND
*****************************.

!Stop !NBlocks.
!ENDDEFINE /* !Bootvar.

SET MPR ON.
!Bootvar.
SET MPR NO.

*** END OF BOOTVARE_V21.SPS PROGRAM  ***.
