*                                    WARNING

* The Government of Canada (Statistics Canada) is the owner of all intellectual
* property rights (including copyright) in this software.  Subject to the terms below,
* you are granted a non-exclusive and non-transferable licence to use this software.
*
* This software is provided "as-is", and the owner makes no warranty, either express
* or implied, including but not limited to, warranties of merchantability and fitness
* for any particular purpose.  In no event will the owner be liable for any indirect,
* special, consequential or other similar damages.  This agreement will terminate
* automatically without notice to you if you fail to comply with any term of this
* agreement.

********************************************************************************************************************
* Date:      April 2002  (version 2.0)                                                                                              *
* Modified: June 2002  (version 2.1)                                                                                              *
* Modified: Feb 2003  (version 2.1)                                                                                              *
*                                                                                                                                               *
* Changes from version 2.0:                                                                                                        *
*                                                                                                                                               *
*    1- In the macro PREPARE, the format of the variable SAMPLEID (used with CCHS) was            *
*        changed from 15 to 20 characters, to reflect the actual size of this variable in the data.           *
*    2- In the macro T_R_DR of version 2.0, an invalid result was produced when the name of the       *
*        denominator variable of a ratio was a substring of the numerator variable (ex: if the                  *   
*        numerator was  "cancer_m" and the denominator was "cancer").                                          *  
*    3- The macro T_R_DR  was corrected to better deal with the cases where either the numerator   *
*        or the denominator had only missing values for one or several classes (values of the               *               
*        breakdown variable(s)).                                                                                                      *
*                                                                                                                                               *
********************************************************************************************************************.

****************************************************************************************************************
***                                                                                                                                    ***
***                                                    MACROE_V21.SAS                                                   ***
***                                                                                                                                     ***
*** This program calculates variance estimates using the bootstrap weights for different types   ***
*** of estimators.  Using SAS Macros, this program can calculate variance estimates for          ***
*** totals, ratios and differences between ratios.  It can also calculate variance estimates for     ***
*** the parameters of a regression or a logistic regression.  This program can also be               ***
*** customized for other types of analyses.                                                                           ***
***                                                                                                                                     ***
*** This program contains the macros that are necessary to use the BOOTVARE_V21.SPS     ***
*** program.                                                                                                                       ***
***                                                                                                                                     ***
*** This program is automatically called by BOOTVARE_V21.SPS and NO MODIFICATIONS   ***
*** SHOULD BE MADE BY THE USER (except for specific cases mentioned in                      ***
*** BOOTVARE_V21.SPS)                                                                                                 ***
***                                                                                                                                    ***
****************************************************************************************************************.

*************************************************************************************************************************
***      SECTION 1:    PRELIMINARY MACROS  (called by principal macros in section 2)                   ***
*************************************************************************************************************************.

**********************************************************************************************************
***  The MACRO "PREPARE" prepares the analysis file and the bootstrap weight file in   ***
***  order to use the following macros                                                                           ***
**********************************************************************************************************.

DEFINE !Prepare(B=!CHAREND('/')/ Classes=!CHAREND('/')/  mfile=!CHAREND('/')/  bwsav=!CHAREND('/') /Keys=!CMDEND)


GET FILE=!QUOTE(!Mfile).
SORT CASES BY !Keys.
display dictionary.
SAVE OUTFILE='C:\Temp\SPSS_Mfile.sav'.


!Let !BSW=!Concat('bsw1 TO bsw',!B)
get sas data=!quote(!bwsav).

display dictionary.

SORT CASES BY !Keys.
SAVE OUTFILE='C:\Temp\SPSS_bsw.sav'  /keep=!keys fwgt !BSW. 

****.
!IF (!B=2000)   !THEN  !LET !NBlocks=20 !LET !BlkSize=100
!ELSE !IF (!B=1000)   !THEN  !LET !NBlocks=20 !LET !BlkSize=50
!ELSE !IF (!B=500)   !THEN  !LET !NBlocks=10 !LET !BlkSize=50 
!ELSE !IF (!B=100)   !THEN  !LET !NBlocks=10 !LET !BlkSize=10 
!ELSE   !LET !NBlocks=1 !LET !BlkSize=!B
!IFEND  !IFEND  !IFEND  !IFEND
!IF  (!B<=10)   !THEN !LET !map=!UNQUOTE('/MAP') !IFEND
****.


!LET !BSW=!CONCAT('bsw1 TO bsw',!B)
!LET !Ilist=''
!DO !I=1 !TO !B !BY !BLKSIZE
   !LET !I1list=!CONCAT(!Ilist,' ',!I)
   !LET !Ilist=!I1list
!DOEND
!LET !Ilist=''
!DO !I=!BLKSIZE !TO !B !BY !BLKSIZE
   !LET !I2list=!CONCAT(!Ilist,' ',!I)
   !LET !Ilist=!I2list
!DOEND

NEW FILE.
INPUT PROGRAM.
NUMERIC !IF (!Classes~='') !THEN !Classes !IFEND Id(F11.6) yhat(F11.2) bhat(F8.5) odds(F9.5)
        wald(F9.4) pvalue(PCT6.2) bs_sd(F11.2) bs_cv(F7.2) cil95 ciu95(F11.2) start(TIME10.1).
!start.


MATCH FILES
	FILE='C:\Temp\SPSS_Mfile.sav' / IN=Mfile/
 	FILE='C:\Temp\SPSS_bsw.sav'/
	BY !Keys. 
SELECT IF Mfile.

COMPUTE Id=$CASENUM.
SORT CASES BY !Classes Id.
XSAVE OUTFILE 'C:\Temp\SPSS_Mfile'/ DROP=Mfile !Keys !BSW !map.

!LET !I1Temp=!I1List
!LET !I2Temp=!I2List
!DO !I=1 !TO !NBlocks
   !LET !I1=!HEAD(!I1Temp)
   !LET !I2=!HEAD(!I2Temp)
   	SAVE OUTFILE !QUOTE(!CONCAT(!UNQUOTE('C:\Temp\SPSS_BSWfile'),!I))/
   	KEEP=!Classes Id !CONCAT(BSW,!I1) TO !CONCAT(BSW,!I2) !map.
   !LET !I1Temp=!TAIL(!I1Temp)
   !LET !I2Temp=!TAIL(!I2Temp)
!DOEND
EXECUTE.

!ENDDEFINE. /* !Prepare*****************.
 


************************************************************************************
***             MACROS USED INSIDE PRINCIPAL MACROS               ***
************************************************************************************.


DEFINE !Dummy(!POS !TOKENS(1)/!POS !CHAREND('/')/Dep=!TOKENS(1)/Indep=!CMDEND)
!LET !B=!1
!LET !Classes=!2
!LET !BSW=!CONCAT('bsw1 TO bsw',!B)
!IF (!Classes~='') !THEN
+	GET FILE='C:\Temp\SPSS_Mfile' /map.
+	AGGREGATE OUTFILE=*/BREAK=!Classes/n=N.
+	WRITE OUTFILE='C:\Temp\SPSS_Dummy2' NOTABLE/!Classes.
+	EXECUTE.
!IFEND
NEW FILE.
INPUT PROGRAM.
+	STRING beta(A8).
+	NUMERIC !IF (!Classes~='') !THEN !Classes !IFEND fwgt !BSW !Dep !Indep(F1).
+	LEAVE !Classes fwgt TO !Dep.
+	VECTOR BSW=!BSW.
+	COMPUTE fwgt=1.
+	LOOP #k=1 TO (!B).
+		COMPUTE BSW(#k)=1.
+	END LOOP.
+	COMPUTE !Dep=0.
	!DO !I !IN (!Indep)
+		COMPUTE !I=0.
	!DOEND
+	!IF (!Classes~='') !THEN DATA LIST FILE='C:\Temp\SPSS_Dummy2'  FREE /!classes  !IFEND
+	COMPUTE beta='Const_'..
+	END CASE.
	!DO !I !IN (!Indep)
+		COMPUTE beta=!QUOTE(!I).
+		COMPUTE !I=1.
+		END CASE.
	!DOEND
+	!IF (!Classes='') !THEN END FILE !IFEND
END INPUT PROGRAM.
RECODE !Indep(SYSMIS=0).
SAVE OUTFILE='C:\Temp\SPSS_Dummy'.
!ENDDEFINE /* !Dummy ******************************************************************************.


DEFINE !Results(!POS !CHAREND(';')/!POS !CHAREND(';')/!POS !CHAREND(';')/!POS !CHAREND(';')/
                !POS !CHAREND(';')/!POS !CHAREND(';')/!POS !CHAREND(';')/!POS !CMDEND)
!LET !File=!QUOTE(!CONCAT(!UNQUOTE('C:\Temp\'),!8,.,sav)).
!LET !Var=beta
!IF (!1=6) !THEN !LET !map=!UNQUOTE('/MAP') !IFEND
!IF (!7='Linear Regression' | !7='General Linear Model') !THEN
+	COMPUTE bs_sd=SQRT(!2*(!1-1)*(bs_sd**2)/!1).
!ELSE 
+	COMPUTE bs_sd=SQRT(!2*(!1-1)*VAR(!UNQUOTE(!6))/!1).
	!IF (!7~='Linear Regression -M') !THEN !LET !Var=variable !IFEND
!IFEND
COMPUTE bs_cv=ABS(RND(10000*bs_sd/!5)/100).
COMPUTE cil95=!5-1.96*bs_sd.
COMPUTE ciu95=!5+1.96*bs_sd.

!IF (!7='Logistic Regression') !THEN
+	COMPUTE wald=(bhat/bs_sd)**2.
+	COMPUTE pvalue=1-CDF.CHISQ(wald,1).
+	COMPUTE odds=EXP(bhat).
+	COMPUTE cil95=EXP(cil95).
+	COMPUTE ciu95=EXP(ciu95).
	!LET !Var=beta
	!LET !LGstats='odds wald pvalue'
!ELSE
	!IF (!7='Ratio' | !7='Diff Ratio') !THEN
+		DO REPEAT R=!5 bs_sd cil95 ciu95.
+			COMPUTE R=100*R.
+		END REPEAT.
	!IFEND
	!LET !LGstats=!NULL
!IFEND

STRING type(A20) /variable(A35).
COMPUTE type=!QUOTE(!7).
COMPUTE variable=!QUOTE(!4).
EXECUTE.
MATCH FILES FILE=* /FILE='C:\Temp\SPSS_PartTime' !map.
COMPUTE #stop=$TIME.
COMPUTE time=#stop-start.
COMPUTE start=#stop.
SAVE OUTFILE 'C:\Temp\SPSS_PartTime2'/KEEP=start !map.
ADD FILES FILE=!File/ RENAME=(start=start1)/FILE=*/ RENAME=(time=start1)/
	KEEP=!3 type !Var !5 !LGstats bs_sd bs_cv cil95 ciu95 start1 !map.
SELECT IF LENGTH(RTRIM(type))>0.
SAVE OUTFILE=!File/RENAME=(start1=start) !map.
!ENDDEFINE /* !Results ***************************************************************************
.


DEFINE !Begin(!POS !CHAREND(';')/!POS !CHAREND(';')/!POS !CHAREND(';')/!POS !CMDEND)
NEW FILE.
INPUT PROGRAM.
+	COMPUTE start=$TIME.
+	END CASE.
+	END FILE.
END INPUT PROGRAM.
EXECUTE.
PRESERVE.
SET PRI NO.
PRINT /'--------------------------------------------'
      /'Type:      ' !QUOTE(!4)
      /'Bssz:      ' !QUOTE(!1)
      /'Multi:     ' !QUOTE(!2)
      /'Variables: ' !QUOTE(!3)
      /'--------------------------------------------'.
RESTORE.
SAVE OUTFILE 'C:\Temp\SPSS_PartTime'.
!ENDDEFINE /* !Begin ******************************************************************************.


DEFINE !Start()
+	COMPUTE start=$TIME.
+	END CASE.
+	END FILE.
END INPUT PROGRAM.
EXECUTE.
XSAVE OUTFILE 'c:\temp\SPSS_TotTime' /KEEP=start.
XSAVE OUTFILE 'c:\temp\SPSS_All_Tots.sav'.
XSAVE OUTFILE 'c:\temp\SPSS_All_Regs.sav'.
EXECUTE.
!ENDDEFINE. /* !Start *****************************************************************************.


DEFINE !Stop(NBlocks=!TOKENS(1))
GET FILE 'C:\Temp\SPSS_TotTime'.
COMPUTE stop=$TIME.
COMPUTE time=stop-start.
PRESERVE.
SET PRI NO.
PRINT / '------------------------------------------------'
      / 'Actual Time:            ' stop(DATETIME24.3)
      / 'Starting Time:          ' start(DATETIME24.3)
      / 'Total Elapsed Time to run Program:  ' time(TIME12.3)
      / '------------------------------------------------'//.
EXECUTE.
RESTORE.
!ENDDEFINE. /* !Stop ******************************************************************************.


DEFINE !Print_T(!POS !CMDEND)
GET FILE 'C:\Temp\SPSS_All_Tots.sav' /RENAME=(start=time).
SORT CASES BY !IF (!1~='') !THEN !1(A) !IFEND type(D).
STRING Type.(A10).
COMPUTE Type.=SUBSTR(Type,1,10).
FORMATS time(F7.1).
LIST !1 Type. variable TO ciu95  /FORMAT=SINGLE.
!ENDDEFINE /* !Print_T ****************************************************************************.


DEFINE !Print_R(!POS !CMDEND)
GET FILE 'C:\Temp\SPSS_All_Regs.sav' /RENAME=(start=time).
*FORMATS bhat bs_sd cil95 ciu95(F9.5) pvalue(F10.8) wald(F9.4) odds(F9.5) time(F9.1).
LIST !1 Type beta TO ciu95  /FORMAT=SINGLE.
!ENDDEFINE /* !Print_R ****************************************************************************.



*************************************************************************************
***             MACROS THAT CALL PRINCIPAL MACROS                    ***
*************************************************************************************.


DEFINE !Total(!POS !TOKENS(1)/!POS !CHAREND('/')/Var=!CMDEND)
!LET !B=!1 !LET !Classes=!2
!let !R=1
****.
!IF (!B=2000)   !THEN  !LET !NBlocks=20 !LET !BlkSize=100
!ELSE !IF (!B=1000)   !THEN  !LET !NBlocks=20 !LET !BlkSize=50
!ELSE !IF (!B=500)   !THEN  !LET !NBlocks=10 !LET !BlkSize=50 
!ELSE !IF (!B=100)   !THEN  !LET !NBlocks=10 !LET !BlkSize=10 
!ELSE   !LET !NBlocks=1 !LET !BlkSize=!B
!IFEND  !IFEND  !IFEND  !IFEND
!IF  (!B<=10)   !THEN !LET !map=!UNQUOTE('/MAP') !IFEND
****.
!T_R_DR  !B !R !NBlocks !Classes/ Varlist=!Var.
!ENDDEFINE /*!Total *******************************************************************************.

DEFINE !Ratio(!POS !TOKENS(1)/!POS !CHAREND('/')/Varlist=!CMDEND)
!LET !B=!1 !LET !Classes=!2
!let !R=1
****.
!IF (!B=2000)   !THEN  !LET !NBlocks=20 !LET !BlkSize=100
!ELSE !IF (!B=1000)   !THEN  !LET !NBlocks=20 !LET !BlkSize=50
!ELSE !IF (!B=500)   !THEN  !LET !NBlocks=10 !LET !BlkSize=50 
!ELSE !IF (!B=100)   !THEN  !LET !NBlocks=10 !LET !BlkSize=10 
!ELSE   !LET !NBlocks=1 !LET !BlkSize=!B
!IFEND  !IFEND  !IFEND  !IFEND
!IF  (!B<=10)   !THEN !LET !map=!UNQUOTE('/MAP') !IFEND
****.
!T_R_DR  !B !R !NBlocks !Classes/ Varlist=!Varlist.
!ENDDEFINE /*!Ratio *******************************************************************************.

DEFINE !Diffrat(!POS !TOKENS(1)/!POS !CHAREND('/')/Varlist=!CMDEND)
!LET !B=!1 !LET !Classes=!2
!let !R=1
****.
!IF (!B=2000)   !THEN  !LET !NBlocks=20 !LET !BlkSize=100
!ELSE !IF (!B=1000)   !THEN  !LET !NBlocks=20 !LET !BlkSize=50
!ELSE !IF (!B=500)   !THEN  !LET !NBlocks=10 !LET !BlkSize=50 
!ELSE !IF (!B=100)   !THEN  !LET !NBlocks=10 !LET !BlkSize=10 
!ELSE   !LET !NBlocks=1 !LET !BlkSize=!B
!IFEND  !IFEND  !IFEND  !IFEND
!IF  (!B<=10)   !THEN !LET !map=!UNQUOTE('/MAP') !IFEND
****.
!T_R_DR  !B !R !NBlocks !Classes/ Varlist=!Varlist.
!ENDDEFINE /*!Diffrat *******************************************************************************.

DEFINE !Pctile(!POS !TOKENS(1)/!POS !CHAREND('/')/Varlist=!CMDEND)
!LET !B=!1 !LET !Classes=!2
!let !R=1
****.
!IF (!B=2000)   !THEN  !LET !NBlocks=20 !LET !BlkSize=100
!ELSE !IF (!B=1000)   !THEN  !LET !NBlocks=20 !LET !BlkSize=50
!ELSE !IF (!B=500)   !THEN  !LET !NBlocks=10 !LET !BlkSize=50 
!ELSE !IF (!B=100)   !THEN  !LET !NBlocks=10 !LET !BlkSize=10 
!ELSE   !LET !NBlocks=1 !LET !BlkSize=!B
!IFEND  !IFEND  !IFEND  !IFEND
!IF  (!B<=10)   !THEN !LET !map=!UNQUOTE('/MAP') !IFEND
****.

!LET !Ilist=''
!DO !I=1 !TO !B !BY !BLKSIZE
   !LET !I1list=!CONCAT(!Ilist,' ',!I)
   !LET !Ilist=!I1list
!DOEND
!LET !Ilist=''
!DO !I=!BLKSIZE !TO !B !BY !BLKSIZE
   !LET !I2list=!CONCAT(!Ilist,' ',!I)
   !LET !Ilist=!I2list
!DOEND

!Pctile2 !B !R !I1List/ !I2List/ !NBlocks !Classes/ Varlist=!Varlist.
!ENDDEFINE /*!Pctile *******************************************************************************.

DEFINE !Regress(!POS !TOKENS(1)/!POS !CHAREND('/')/Dep=!TOKENS(1)/Indep=!CMDEND)
!LET !B=!1 !LET !Classes=!2
!let !R=1
****.
!IF (!B=2000)   !THEN  !LET !NBlocks=20 !LET !BlkSize=100
!ELSE !IF (!B=1000)   !THEN  !LET !NBlocks=20 !LET !BlkSize=50
!ELSE !IF (!B=500)   !THEN  !LET !NBlocks=10 !LET !BlkSize=50 
!ELSE !IF (!B=100)   !THEN  !LET !NBlocks=10 !LET !BlkSize=10 
!ELSE   !LET !NBlocks=1 !LET !BlkSize=!B
!IFEND  !IFEND  !IFEND  !IFEND
!IF  (!B<=10)   !THEN !LET !map=!UNQUOTE('/MAP') !IFEND
****.

!LET !Ilist=''
!DO !I=1 !TO !B !BY !BLKSIZE
   !LET !I1list=!CONCAT(!Ilist,' ',!I)
   !LET !Ilist=!I1list
!DOEND
!LET !Ilist=''
!DO !I=!BLKSIZE !TO !B !BY !BLKSIZE
   !LET !I2list=!CONCAT(!Ilist,' ',!I)
   !LET !Ilist=!I2list
!DOEND

!Regres2 !B !R !I1List/ !I2List/ !NBlocks !Classes/ Dep =!Dep  Indep = !Indep.
!ENDDEFINE /*!Regress *******************************************************************************.


DEFINE !Log_Reg(!POS !TOKENS(1)/!POS !CHAREND('/')/Cri=!TOKENS(1)/Dep=!TOKENS(1)/Indep=!CMDEND)
!LET !B=!1 !LET !Classes=!2
!let !R=1
****.
!IF (!B=2000)   !THEN  !LET !NBlocks=20 !LET !BlkSize=100
!ELSE !IF (!B=1000)   !THEN  !LET !NBlocks=20 !LET !BlkSize=50
!ELSE !IF (!B=500)   !THEN  !LET !NBlocks=10 !LET !BlkSize=50 
!ELSE !IF (!B=100)   !THEN  !LET !NBlocks=10 !LET !BlkSize=10 
!ELSE   !LET !NBlocks=1 !LET !BlkSize=!B
!IFEND  !IFEND  !IFEND  !IFEND
!IF  (!B<=10)   !THEN !LET !map=!UNQUOTE('/MAP') !IFEND
****.

!LET !Ilist=''
!DO !I=1 !TO !B !BY !BLKSIZE
   !LET !I1list=!CONCAT(!Ilist,' ',!I)
   !LET !Ilist=!I1list
!DOEND
!LET !Ilist=''
!DO !I=!BLKSIZE !TO !B !BY !BLKSIZE
   !LET !I2list=!CONCAT(!Ilist,' ',!I)
   !LET !Ilist=!I2list
!DOEND

!LogReg2 !B !R !I1List/ !I2List/ !NBlocks !Classes/ Cri=!Cri Dep =!Dep  Indep = !Indep.
!ENDDEFINE /*!Log_Reg *******************************************************************************.


                         

***************************************************************************************************************************************************
***                         SECTION 2:         PRINCIPAL MACROS FOR VARIANCE ESTIMATION                                               ***
***************************************************************************************************************************************************.

******************************************************************************************
***               TOTALS, RATIOS AND DIFFERENCES OF RATIO                ***
******************************************************************************************.

DEFINE !T_R_DR(!POS !TOKENS(1)/!POS !TOKENS(1)/!POS !TOKENS(1)/!POS !CHAREND('/')/Varlist=!CMDEND)
!LET !B=!1 !LET !NBlocks=!3
!LET !R=!2 !LET !Classes=!4
!LET !BSW=!CONCAT('bsw1 TO bsw',!B)
!LET !Y=!CONCAT('y1_1 TO y1_',!B)
!LET !Var1=!HEAD(!Varlist)
!LET !Var2=!HEAD(!TAIL(!Varlist))
!LET !Var3=!HEAD(!TAIL(!TAIL(!Varlist)))
!LET !Var4=!HEAD(!TAIL(!TAIL(!TAIL(!Varlist))))
!IF             (!Var4~=!NULL) !THEN !LET !Type='Diff Ratio'
!ELSE !IF       (!Var3~=!NULL) !THEN !LET !Type='**ERROR**'
      !ELSE !IF (!Var2~=!NULL) !THEN !LET !Type='Ratio'
            !ELSE                    !LET !Type='Total'
            !IFEND
      !IFEND
!IFEND
!IF (!B<=10) !THEN !LET !map=!UNQUOTE('/MAP') !IFEND


!Begin !B; !R; !Varlist; !Type.
GET FILE='C:\Temp\SPSS_Mfile' !map.


!DO !IBlock=1 !TO !NBlocks
+	MATCH FILES FILE=*/FILE=!QUOTE(!CONCAT('C:\Temp\SPSS_BSWfile',!IBlock))/BY !Classes Id !map.
!DOEND



!IF (!Classes='') !THEN COMPUTE Dummy=1 !LET !Classes=Dummy !IFEND

!IF (!Type='Total') !THEN

COMPUTE FWGT=FWGT*!Var1.
!DO !I=1 !TO !B
+          COMPUTE !CONCAT(bsw,!I) = !CONCAT(bsw,!I)  *!Var1.
!DOEND

+	SELECT IF !Var1>0.
+	AGGREGATE OUTFILE=*/PRESORTED/BREAK=!Classes/y1_0 !Y=SUM(fwgt !BSW).
!ELSE
	!DO !VAR !IN (!Varlist)
+		TEMPORARY.
+                       !IF  (!VAR=!VAR1) !THEN !LET !ITER=1 !IFEND
+                       !IF  (!VAR=!VAR2) !THEN !LET !ITER=2 !IFEND
+                       !IF  (!VAR=!VAR3) !THEN !LET !ITER=3 !IFEND
+                       !IF  (!VAR=!VAR4) !THEN !LET !ITER=4 !IFEND
+		weight by !VAR.
+		AGGREGATE OUTFILE=!QUOTE(!CONCAT(!UNQUOTE('C:\Temp\SPSS_'),!VAR))/PRESORTED/BREAK=!Classes/
			!CONCAT(y,!ITER,_,0) TO !CONCAT(y,!ITER,_,!B)=SUM(fwgt !BSW).
	!DOEND
+	weight off.
!IFEND
!IF (!Classes=Dummy) !THEN !LET !Classes='' !IFEND

!IF (!Type~='Total') !THEN
	!IF (!Type='Ratio') !THEN
+		MATCH FILES FILE=!QUOTE(!CONCAT(!UNQUOTE('C:\Temp\SPSS_'),!Var1))/
		            FILE=!QUOTE(!CONCAT(!UNQUOTE('C:\Temp\SPSS_'),!Var2)) !IF (!Classes~='') !THEN /BY !Classes !IFEND !map.

+		VECTOR y=y1_0 TO !CONCAT(y1_,!B)/
		       x=y2_0 TO !CONCAT(y2_,!B).
+		LOOP #i=1 TO !B+1.
+			COMPUTE y(#i)=y(#i)/x(#i).
+		END LOOP.
	!ELSE
+		
		!IF (!VAR2=!VAR4) !THEN

+			MATCH FILES FILE=!QUOTE(!CONCAT(!UNQUOTE('C:\Temp\SPSS_'),!Var1))/
			            FILE=!QUOTE(!CONCAT(!UNQUOTE('C:\Temp\SPSS_'),!Var2))/
			            FILE=!QUOTE(!CONCAT(!UNQUOTE('C:\Temp\SPSS_'),!Var3))  !IF (!Classes~='') !THEN /BY !Classes !IFEND !map.
+			VECTOR y=y1_0 TO !CONCAT(y1_,!B)/
			       x=y4_0 TO !CONCAT(y4_,!B)/
			       w=y3_0 TO !CONCAT(y3_,!B).
+			LOOP #i=1 TO !B+1.
+				COMPUTE y(#i)=(y(#i)/x(#i))-(w(#i)/x(#i)).
+			END LOOP.

		!ELSE

+			MATCH FILES FILE=!QUOTE(!CONCAT(!UNQUOTE('C:\Temp\SPSS_'),!Var1))/
			            FILE=!QUOTE(!CONCAT(!UNQUOTE('C:\Temp\SPSS_'),!Var2))/
			            FILE=!QUOTE(!CONCAT(!UNQUOTE('C:\Temp\SPSS_'),!Var3))/
			            FILE=!QUOTE(!CONCAT(!UNQUOTE('C:\Temp\SPSS_'),!Var4))  !IF (!Classes~='') !THEN /BY !Classes !IFEND !map.
+			VECTOR y=y1_0 TO !CONCAT(y1_,!B)/
			       x=y2_0 TO !CONCAT(y2_,!B)/
			       w=y3_0 TO !CONCAT(y3_,!B)/
	          	   	       v=y4_0 TO !CONCAT(y4_,!B).
+			LOOP #i=1 TO !B+1.
+				COMPUTE y(#i)=(y(#i)/x(#i))-(w(#i)/v(#i)).
+			END LOOP.
		!IFEND
	!IFEND
!ELSE
+	VECTOR y=y1_0 TO !CONCAT(y1_,!B).
!IFEND

RENAME VARIABLES (y1_0=yhat).

!Results !B; !R; !Classes; !Varlist; yhat; !Y; !Type; SPSS_All_Tots.
!ENDDEFINE /* !T_R_DR ****************************************************************************.


**************************************************************************
***                     PERCENTILES                                    ***
**************************************************************************.

DEFINE !Pctile2(!POS !TOKENS(1)/!POS !TOKENS(1)/!POS !CHAREND('/')/!POS !CHAREND('/')/
                !POS !TOKENS(1)/!POS !CHAREND('/')/Varlist=!CMDEND)
!Let !B=!1 !LET !I1List=!3 !LET !NBlocks=!5
!LET !R=!2 !LET !I2List=!4 !LET !Classes=!6
!LET !Type='Percentile'
!LET !Var1=!HEAD(!Varlist)
!LET !pct=!TAIL(!Varlist)
!IF (!B<=10) !THEN !LET !map=!UNQUOTE('/MAP') !IFEND

!Begin !B; !R; !Varlist; !Type.
GET FILE='C:\Temp\SPSS_Mfile' !map.
SELECT IF (NOT (MISSING(!Var1))).

!IF (!Classes='') !THEN COMPUTE DUMMY=1 !LET !Classes=Dummy !IFEND

SPLIT FILE BY !Classes.

SORT CASES BY !Classes !Var1.

SPLIT FILE
  SEPARATE BY !Classes.

CREATE
 /cumsum_f=CSUM(fwgt).

AGGREGATE
  /OUTFILE='C:\temp\SPSS_total'
  /BREAK=!Classes
  /cumsum_1 = MAX(cumsum_f).

MATCH FILES /FILE=*
 /TABLE='C:\temp\SPSS_total'
 /BY !Classes.
EXECUTE.

COMPUTE percent = cumsum_f / cumsum_1 .
EXECUTE .

COMPUTE prctlag = LAG(percent) .
EXECUTE .

FILTER OFF.
USE ALL.
SELECT IF(percent >= !Pct /100 AND prctlag <!Pct /100).
NUMERIC Yhattemp(F11.2).
COMPUTE yhattemp=!Var1+0.00.
COMPUTE yhat=yhattemp.

SAVE OUTFILE ' C:\temp\spss_bsw' /KEEP = !Classes yhat.

!LET !I1List=!I1List                      /* Double loop on bootstrap weights
!LET !I2List=!I2List
!LET !I1=!HEAD(!I1List)
!LET !I2=!HEAD(!I2List)

NEW FILE.
!IF (!Classes~=Dummy) !THEN MATCH FILES FILE='C:\Temp\SPSS_BSWfile1'/FILE='C:\Temp\SPSS_Mfile'/
 	BY !Classes Id/KEEP=!Classes Id BSW1 TO !CONCAT(BSW,!I2) !Var1 fwgt !map.
!ELSE MATCH FILES FILE='C:\Temp\SPSS_BSWfile1'/FILE='C:\Temp\SPSS_Mfile'/
 	BY Id/KEEP=Id BSW1 TO !CONCAT(BSW,!I2) !Var1 fwgt !map.
COMPUTE DUMMY=1.
!IFEND
SAVE OUTFILE 'c:\temp\SPSS_tempfile' / KEEP=ALL .

!LET !I1List=!TAIL(!I1List)
!LET !I2List=!TAIL(!I2List)

!DO !IBlock=2 !TO !NBlocks
!LET !I1=!HEAD(!I1List)
!LET !I2=!HEAD(!I2List)
+    !IF (!Classes~=Dummy) !THEN MATCH FILES FILE=*/FILE=!QUOTE(!CONCAT('C:\Temp\SPSS_BSWfile',!IBlock))/
 	BY !Classes Id/KEEP=!Classes Id BSW1 TO !CONCAT(BSW,!I2) !Var1 fwgt !map.
+    !ELSE MATCH FILES FILE=*/FILE=!QUOTE(!CONCAT('C:\Temp\SPSS_BSWfile',!IBlock))/
 	BY Id/KEEP=Id BSW1 TO !CONCAT(BSW,!I2) !Var1 fwgt !map.
+    !IFEND
+    EXECUTE.
+    SAVE OUTFILE 'c:\temp\SPSS_tempfile' / KEEP=ALL .

!LET !I1List=!TAIL(!I1List)
!LET !I2List=!TAIL(!I2List)
!DOEND

+    GET FILE='C:\Temp\SPSS_tempfile' / .

+    SPLIT FILE BY !Classes. 

+    SORT CASES BY !Classes !Var1.

+ SAVE OUTFILE 'c:\temp\SPSS_tempfile' / KEEP=ALL .

!DO !I=1 !TO !B

+  GET FILE='C:\Temp\SPSS_tempfile' /  .
SELECT IF (NOT (MISSING(!Var1))).

+   SPLIT FILE
      SEPARATE BY !Classes.

+    CREATE
      /cumsum_f=CSUM(!CONCAT(BSW,!I)).

+    AGGREGATE
      /OUTFILE= 'C:\Temp\SPSS_total'
      /BREAK=!Classes
      /cumsum_1 = MAX(cumsum_f).

+   MATCH FILES /FILE=*
      /TABLE= 'C:\Temp\SPSS_total'
     /BY !Classes.


+   COMPUTE percent = cumsum_f / cumsum_1 .

+   COMPUTE prctlag = LAG(percent) .

+   SAVE OUTFILE 'c:\temp\SPSS_dummy1' /KEEP=all.

+   FILTER OFF.
+   USE ALL.
+   SELECT IF(percent >= !Pct /100 AND prctlag <!Pct /100).

+   SAVE OUTFILE 'c:\temp\SPSS_dummy2' /KEEP=all.

+   RENAME VARIABLES (!Var1=!CONCAT(y_,!I)).
+   SAVE OUTFILE !QUOTE(!CONCAT(!UNQUOTE('C:\Temp\SPSS_BSW'),!I)) /KEEP = !Classes !CONCAT(y_,!I).

+   MATCH FILES FILE='C:\Temp\SPSS_BSW'/FILE=!QUOTE(!CONCAT('C:\Temp\SPSS_BSW',!I))/ BY !CLASSES / .
+   SAVE OUTFILE='C:\Temp\SPSS_BSW' / .

!DOEND

GET FILE='C:\Temp\SPSS_BSW' / .

!LET !Y=!CONCAT('y_1 TO y_',!B)

!IF (!Classes=Dummy) !THEN !LET !Classes='' !IFEND

!Results !B; !R; !Classes; !Varlist; yhat; !Y; !Type; SPSS_All_Tots.

!DO !I=1 !TO !B
+   ERASE FILE=!QUOTE(!CONCAT('C:\Temp\SPSS_BSW',!I)).
!DOEND
ERASE FILE = 'C:\temp\SPSS_total'.
ERASE FILE = 'C:\temp\SPSS_dummy1'.
ERASE FILE = 'C:\temp\SPSS_dummy2'.
NEW FILE.


!ENDDEFINE /* !Pctile2 ***************************************************************************.




*******************************************************************************
***                            LINEAR  REGRESSION                            ***
*******************************************************************************.


DEFINE !Regres2(!POS !TOKENS(1)/!POS !TOKENS(1)/!POS !CHAREND('/')/!POS !CHAREND('/')/
                !POS !TOKENS(1)/!POS !CHAREND('/')/Dep=!TOKENS(1)/Indep=!CMDEND)
!LET !B=!1 !LET !I1List=!3 !LET !NBlocks=!5
!LET !R=!2 !LET !I2List=!4 !LET !Classes=!6
!LET !BSW=!CONCAT('bsw1 to bsw',!B)
!LET !Type='Linear Regression'
!LET !Vars=!CONCAT('Y: ',!Dep,', X: ',!Indep)
!IF (!B<=10) !THEN !LET !map=!UNQUOTE('/MAP') !IFEND

!Begin !B; !R; !Vars; !Type.
GET FILE='C:\Temp\SPSS_Mfile' !map.
!IF (!Classes~='') !THEN SPLIT FILE BY !Classes !IFEND
REGRESSION VAR=!Dep !Indep/REG=fwgt/STA=COEFF/DEP=!Dep/MET=ENTER/OUT=CORB ('C:\Temp\SPSS_BSW').

!LET !I1List=!I1List                      /* Double loop on bootstrap weights
!LET !I2List=!I2List
!DO !IBlock=1 !TO !NBlocks
	!LET !I1=!HEAD(!I1List)
	!LET !I2=!HEAD(!I2List)
+	MATCH FILES FILE=!QUOTE(!CONCAT('C:\Temp\SPSS_BSWfile',!IBlock))/FILE='C:\Temp\SPSS_Mfile'/
		BY !Classes Id/KEEP=!Classes Id !CONCAT(BSW,!I1) TO !CONCAT(BSW,!I2)!Dep !Indep !map.
+	!IF (!Classes~='') !THEN SPLIT FILE BY !Classes !IFEND
	!DO !I=!I1 !TO !I2
+		REGRESSION VAR=!Dep !Indep/REG=!CONCAT(bsw,!I)/STA=COEFF/DEP=!Dep/MET=ENTER/
			OUT=CORB(!QUOTE(!CONCAT(!UNQUOTE('C:\Temp\SPSS_BSW'),!I))).
	!DOEND
	!LET !I1List=!TAIL(!I1List)
	!LET !I2List=!TAIL(!I2List)
!DOEND

GET FILE='C:\Temp\SPSS_BSW'/ !map.
!IF (!Classes='') !THEN
+	SELECT IF (ROWTYPE_='EST').
+	SAVE OUT='C:\Temp\SPSS_BSW0'/DROP=ROWTYPE_ VARNAME_ !map.
+	GET FILE='C:\Temp\SPSS_BSW1'/!map.
+	SELECT IF (ROWTYPE_='EST').
+	!DO !I=2 !TO !B
+  	ADD FILES FILE=*/FILE=!QUOTE(!CONCAT('C:\Temp\SPSS_BSW',!I))/ !map.
+		SELECT IF (ROWTYPE_='EST').
	!DOEND
+	COMPUTE Dummy=1.
+	AGG OUT=*/BREAK=Dummy/CONST_ !Indep =SD(CONST_ !Indep).
+	ADD FILES FILE='C:\Temp\SPSS_BSW0'/FILE=*/DROP=Dummy !map.
+	FLIP.
+	RENAME VARIABLES (Case_lbl Var001 Var002 = beta bhat CONST_).
!ELSE
+	SELECT IF ANY(ROWTYPE_,'COR','EST').
+	TEMPORARY.
+	SELECT IF (ROWTYPE_='COR').
+	SAVE OUT='C:\Temp\SPSS_BSW_beta'/REN VARNAME_=beta/KEEP=!Classes beta !map.
+	SELECT IF (ROWTYPE_='EST').
	!DO !I !IN (!Indep)
+		XSAVE OUT=!QUOTE(!CONCAT('C:\Temp\SPSS_',!I))/KEEP=!Classes !I.
	!DOEND
+	EXECUTE.
	!DO !I !IN (!Indep)
+		ADD FILES FILE=*/FILE=!QUOTE(!CONCAT('C:\Temp\SPSS_',!I))/REN !I=CONST_/
			BY !Classes/KEEP=!Classes CONST_ !map.
	!DOEND
+	MATCH FILES FILE='C:\Temp\SPSS_BSW_beta'/FILE=*/RENAME CONST_=bhat/KEEP=!Classes beta bhat !map.
+	SAVE OUTFILE='C:\Temp\SPSS_BSW' !map.
+	GET FILE='C:\Temp\SPSS_BSW1'/DROP=DEPVAR_ VARNAME_ !map.
+	SELECT IF (ROWTYPE_='EST').
	!DO !I=2 !TO !B
+		ADD FILES FILE=*/FILE=!QUOTE(!CONCAT('C:\Temp\SPSS_BSW',!I))/
			BY !Classes/!map.       
+		SELECT IF (ROWTYPE_='EST').
	!DOEND
+	AGG OUT=*/BREAK=!Classes/CONST_ !Indep =SD(CONST_ !Indep).
	!DO !I !IN (!Indep)
+		XSAVE OUT=!QUOTE(!CONCAT('C:\Temp\SPSS_',!I))/KEEP=!Classes !I !map.
	!DOEND
+	EXECUTE.
	!DO !I !IN (!Indep)
+		ADD FILES FILE=*/FILE=!QUOTE(!CONCAT('C:\Temp\SPSS_',!I))/REN !I=CONST_/
			BY !Classes/KEEP=!Classes CONST_ !map.
	!DOEND
!IFEND

MATCH FILES FILE='C:\Temp\SPSS_BSW'/FILE=*/REN CONST_=bs_sd
	!IF (!Classes~='') !THEN /BY !Classes !IFEND !map.

!Results !B; !R; !Classes; !Dep; bhat; none; !Type; SPSS_All_Regs.
!ENDDEFINE /* !Regres2 ***************************************************************************.


**************************************************************************
***                     LOGISTIC  REGRESSION                          ***
**************************************************************************.


DEFINE !LogReg2(!POS !TOKENS(1)/!POS !TOKENS(1)/!POS !CHAREND('/')/!POS !CHAREND('/')/
                !POS !TOKENS(1)/!POS !CHAREND('/')/Cri=!TOKENS(1)/Dep=!TOKENS(1)/Indep=!CMDEND)
!LET !B=!1 !LET !I1List=!3 !LET !NBlocks=!5
!LET !R=!2 !LET !I2List=!4 !LET !Classes=!6
!LET !BSW=!CONCAT('bsw1 TO bsw',!B)
!LET !Type='Logistic Regression'
!LET !Vars=!CONCAT('Y: ',!Dep,', X: ',!Indep)
!IF (!B<=10) !THEN !LET !map=!UNQUOTE('/MAP') !IFEND

!Begin !B; !R; !Vars; !Type.
!Dummy !B !Classes/ Dep=!Dep Indep=!Indep
ADD FILES FILE=*/IN=dummy/FILE='C:\Temp\SPSS_Mfile'/KEEP=beta !Classes Id fwgt !Dep !Indep/
	!IF (!Classes~='') !THEN BY !Classes/FIRST=first !IFEND !map.
!IF (!Classes~='') !THEN SPLIT FILE BY !Classes !IFEND
TEMPORARY.
SELECT IF fwgt>0.
WEIGHT BY fwgt.
PRESERVE.
SET RES NO.
LOGISTIC REG VARIABLES=!Dep WITH !Indep/SELECT dummy=0/PRINT=SUMMARY/SAVE=PRED(bhat)/
	!IF (!UPCASE(!Cri)=SAS) !THEN CRITERIA=BCON(0.0001) LCON(0) ITERATE(25) EPS(10E-12) !IFEND.
RESTORE.
SELECT IF dummy=1.
SAVE OUTFILE='C:\Temp\SPSS_LogReg'/KEEP=!IF (!Classes~='') !THEN first !Classes !IFEND beta bhat !map.

!LET !I1List=!I1List                      /* Double loop on bootstrap weights
!LET !I2List=!I2List
!DO !IBlock=1 !TO !NBlocks
	!LET !I1=!HEAD(!I1List)
	!LET !I2=!HEAD(!I2List)
+	MATCH FILES FILE=!QUOTE(!CONCAT('C:\Temp\SPSS_BSWfile',!IBlock))/FILE='C:\Temp\SPSS_Mfile'/
		BY !Classes Id/KEEP=!Classes Id !CONCAT(BSW,!I1) TO !CONCAT(BSW,!I2)!Dep !Indep !map.
+	ADD FILES FILE='C:\Temp\SPSS_Dummy'/IN=dummy/FILE=*/!IF (!Classes~='') !THEN BY !Classes/ !IFEND
		KEEP=!Classes Id !CONCAT(BSW,!I1) TO !CONCAT(BSW,!I2)!Dep !Indep !map.
+	!IF (!Classes~='') !THEN SPLIT FILE BY !Classes !IFEND
	!DO !I=!I1 !TO !I2
+		TEMPORARY.
+		SELECT IF !CONCAT('bsw',!I)>0.
+		WEIGHT BY !CONCAT('bsw',!I).
+		PRESERVE.
+		SET RES NO.
+		LOGISTIC REG VARIABLES=!Dep WITH !Indep/SELECT dummy=0/PRINT=SUMMARY/SAVE=PRED(!CONCAT('beta',!I))/
			!IF (!UPCASE(!Cri)=SAS) !THEN CRITERIA=BCON(0.0001) LCON(0) ITERATE(25) EPS(10E-12) !IFEND.
+		RESTORE.
	!DOEND
+	SPLIT FILE OFF.
+	SELECT IF dummy=1.
+	SAVE OUTFILE=!QUOTE(!CONCAT('C:\Temp\SPSS_LogReg',!IBlock))/
		KEEP=!CONCAT(beta,!I1) TO !CONCAT(beta,!I2) !map.
	!LET !I1List=!TAIL(!I1List)
	!LET !I2List=!TAIL(!I2List)
!DOEND

GET FILE='C:\Temp\SPSS_LogReg'.
!DO !IBlock=1 !TO !NBlocks
+	MATCH FILES FILE=*/FILE=!QUOTE(!CONCAT('C:\Temp\SPSS_LogReg',!IBlock)) !map.
!DOEND

NUMERIC w0 w1 TO !CONCAT('w',!B). /* Calculation of the parameter estimates
LEAVE w0 TO !CONCAT('w',!B).
VECTOR w=w0 TO !CONCAT('w',!B).
VECTOR v=bhat TO !CONCAT('beta',!B).
LOOP #i=1 TO (1+!B).
+	COMPUTE v(#i)=LN(v(#i)/(1-v(#i))).
+	DO IF !IF (!Classes~='') !THEN first !ELSE $CASENUM !IFEND =1.
+		COMPUTE w(#i)=v(#i).
+  	ELSE.
+		COMPUTE v(#i)=v(#i)-w(#i).
+	END IF.
END LOOP.

!Results !B; !R; !Classes; !Dep; bhat; beta1 TO !CONCAT(beta,!I); !Type; SPSS_All_Regs.
!ENDDEFINE /* !LogReg2 ***************************************************************************.

