														  
*                                    AVERTISSEMENT

* Le gouvernement du Canada (Statistique Canada) possde tous les droits de proprit
* intellectuelle (y compris le droit d'auteur) relatifs  ce logiciel.  Sous rserve
* des modalits dcrites ci-dessous, vous obtenez une licence non exclusive et non
* transmissible d'utilisation de ce logiciel.
*
* Ce logiciel vous est fourni  tel quel , et le titulaire ne donne aucune garantie
* expresse ou implicite, ce qui comprend les garanties de qualit marchande et de
* convenance  une fin particulire mais ne se limite pas  ces garanties.  En aucune
* circonstance le titulaire ne sera responsable de dommages indirects, particuliers ou
* conscutifs, ou d'autres dommages semblables.  Ce contrat sera automatiquement rsili,
* sans pravis, si vous omettez de respecter une quelconque des modalits du contrat.;

*                                    WARNING

* The Government of Canada (Statistics Canada) is the owner of all intellectual
* property rights (including copyright) in this software.  Subject to the terms below,
* you are granted a non-exclusive and non-transferable licence to use this software.
*
* This software is provided "as-is", and the owner makes no warranty, either express
* or implied, including but not limited to, warranties of merchantability and fitness
* for any particular purpose.  In no event will the owner be liable for any indirect,
* special, consequential or other similar damages.  This agreement will terminate
* automatically without notice to you if you fail to comply with any term of this
* agreement.;

/*
HERE IS THE INTERFACE:
%partitio(domains=); *no partition by default;
%total(dataset=,variable=,nb_wgts=);
%ratio(dataset=,numer=,denom=,nb_wgts=);
%diff_rat(dataset=,numer1=,denom1=,numer2=,denom2=,nb_wgts=);
%regress(dataset=,dependnt=,independ=,nb_wgts=);
%logistic(dataset=,dependnt=,independ=,nb_wgts=);
%startchr;   * these are for measuring time;
%stop_chr;
*/


proc printto new log=dump_log;  run;

%macro chk_ver;
%global __interc;
%if (&sysver >= 8) %then %let __interc=intercept;
%else %let __interc=intercep;
%mend chk_ver;
%chk_ver;


*************************************;

%macro total(dataset=,variable=,nb_wgts=);

*************************************;
   proc datasets library=work;
     delete ytot est;
   run;

proc means data=&dataset noprint;
  var fwgt bsw1-bsw&nb_wgts;
  weight &variable;
&by  class &region;
  output out=ytot
         sum=Sum ybs1-ybs&nb_wgts;
run;

*arrange with the most interresting on top;
proc sort data=ytot; by descending _type_ &region; run;

data est;
  set ytot;
  length variable $ 32;
  bs_var=((&nb_wgts-1)*(var(of ybs1-ybs&nb_wgts)))/&nb_wgts;
  bs_sd=sqrt(bs_var);
  bs_cv=round((bs_sd/Sum)*100,.01);
  cil95=Sum-1.96*bs_sd;
  ciu95=Sum+1.96*bs_sd;
  variable="&variable";
  drop ybs1-ybs&nb_wgts _type_ _freq_;
  label 
  	Sum		="Sum"
  	variable="Variable"
  	bs_var	="Bootstrap Variance"
	bs_sd	="Bootstrap Standard Deviation"
	bs_cv	="Bootstrap Coefficient of Variation"
	cil95	="Lower 95% Confidence Limit"
	ciu95	="Upper 95% Confidence Limit"
	;
run;

proc append data=est base=totals force;
run;


%mend total;


*******************************************;

%macro ratio(dataset=,numer=,denom=,nb_wgts=);

*******************************************;
   proc datasets library=work;
     delete ytot xtot est;
   run;

proc means data=&dataset(where=(&numer^=. and &denom^=.)) noprint;
  var fwgt bsw1-bsw&nb_wgts;
  weight &numer;
&by  class &region;
  output out=ytot
         sum=yhat ybs1-ybs&nb_wgts;
run;

proc means data=&dataset(where=(&numer^=. and &denom^=.)) noprint;
  var fwgt bsw1-bsw&nb_wgts;
  weight &denom;
&by  class &region;
  output out=xtot
         sum=xhat xbs1-xbs&nb_wgts;
run;

data est;
  merge ytot xtot;
  array ybs{&nb_wgts};
  array xbs{&nb_wgts};
  array rbs{&nb_wgts};
  length Numer $ 32;
  length Denom $ 32;
  ratio=((yhat/xhat)*100);
  do i=1 to &nb_wgts;
    rbs{i}=((ybs{i}/xbs{i})*100);
  end;
  bs_var=((&nb_wgts-1)*(var(of rbs1-rbs&nb_wgts)))/&nb_wgts;
  bs_sd=sqrt(bs_var);
  bs_cv=round((bs_sd/ratio)*100,.01);
  cil95=ratio-1.96*bs_sd;
  ciu95=ratio+1.96*bs_sd;
  Numer="&numer";
  Denom="&denom";
  drop 	ybs1-ybs&nb_wgts 
		xbs1-xbs&nb_wgts 
		rbs1-rbs&nb_wgts 
		xhat yhat i _freq_;
  label
  	Numer	="numer"
	Denom	="denom"
	ratio	="Ratio - as Percentage"
  	bs_var	="Bootstrap Variance"
	bs_sd	="Bootstrap Standard Deviation"
	bs_cv	="Bootstrap Coefficient of Variation"
	cil95	="Lower 95% Confidence Limit"
	ciu95	="Upper 95% Confidence Limit"
  ;
run;

*arrange with the most interresting on top;
proc sort data=est out=est(drop=_type_) ; 
by descending _type_ &region; run;

proc append data=est base=ratios force;
run;


%mend ratio;


********************************************************;

%macro diff_rat(dataset=,numer1=,denom1=,
					numer2=,denom2=,nb_wgts=);

********************************************************;

   proc datasets library=work;
     delete ytot xtot yytot xxtot est;
   run;

%local cond_tn;
%let cond_tn= &numer1^=. and &denom1^=. and &numer2^=. and &denom2^=.;

proc means data=&dataset(where=(&cond_tn)) noprint;
  var fwgt bsw1-bsw&nb_wgts;
  weight &numer1;
&by  class &region;
  output out=ytot
         sum=yhat ybs1-ybs&nb_wgts;
run;

proc means data=&dataset(where=(&cond_tn)) noprint;
  var fwgt bsw1-bsw&nb_wgts;
  weight &denom1;
&by  class &region;
  output out=xtot
         sum=xhat xbs1-xbs&nb_wgts;
run;

proc means data=&dataset(where=(&cond_tn)) noprint;
  var fwgt bsw1-bsw&nb_wgts;
  weight &numer2;
&by  class &region;
  output out=yytot
         sum=yyhat yybs1-yybs&nb_wgts;
run;

proc means data=&dataset(where=(&cond_tn)) noprint;
  var fwgt bsw1-bsw&nb_wgts;
  weight &denom2;
&by  class &region;
  output out=xxtot
         sum=xxhat xxbs1-xxbs&nb_wgts;
run;

data est;
  merge ytot xtot yytot xxtot;
  array ybs{&nb_wgts};
  array xbs{&nb_wgts};
  array yybs{&nb_wgts};
  array xxbs{&nb_wgts};
  array drbs{&nb_wgts};
  length Numer1 $ 32;
  length Denom1 $ 32;
  length Numer2 $ 32;
  length Denom2 $ 32;
  diffrat=(((yhat/xhat)-(yyhat/xxhat))*100);
  do i=1 to &nb_wgts;
    drbs{i}=(((ybs{i}/xbs{i})-(yybs{i}/xxbs{i}))*100);
  end;
  bs_var=(((&nb_wgts-1)*(var(of drbs1-drbs&nb_wgts)))/&nb_wgts);
  bs_sd=sqrt(bs_var);
  bs_cv=abs(round((bs_sd/diffrat)*100,.01));
  cil95=diffrat-1.96*bs_sd;
  ciu95=diffrat+1.96*bs_sd;
  Numer1="&numer1";
  Denom1="&denom1";
  Numer2="&numer2";
  Denom2="&denom2";
  drop 	ybs1-ybs&nb_wgts 
		xbs1-xbs&nb_wgts 
		yybs1-yybs&nb_wgts 
		xxbs1-xxbs&nb_wgts 
		drbs1-drbs&nb_wgts
       	xhat yhat xxhat yyhat i _freq_;
  label
  	Numer1	="numer - First Ratio"
	Denom1	="denom - First Ratio"
  	Numer2	="numer - Second Ratio"
	Denom2	="denom - Second Ratio"
	diffrat	="Difference of Percentages"
  	bs_var	="Bootstrap Variance"
	bs_sd	="Bootstrap Standard Deviation"
	bs_cv	="Bootstrap Coefficient of Variation"
	cil95	="Lower 95% Confidence Limit"
	ciu95	="Upper 95% Confidence Limit"
  ;
run;

*arrange with the most interresting on top;
proc sort data=est out=est(drop=_type_); by descending _type_ &region; run;

proc append data=est base=diffrat force;
run;

%mend diff_rat;


*********************************************;

%macro regress(dataset=,dependnt=,independ=,nb_wgts=);

*********************************************;
%local chunk_sz start stop k i;


data _rgr_set;
	set &dataset;
run;

&by proc sort data=_rgr_set; 
&by by &region; 
&by run; 

   proc datasets library=work;
     delete betas betat bsbeta origest est;
   run;

proc reg data=_rgr_set outest=orig(keep=&region &__interc &independ) noprint;
  model &dependnt=&independ;
  weight fwgt;
&by  by &region;
run;

proc transpose data=orig out=origest(drop=_label_) prefix=bhat name=beta;
  var &__interc &independ;
&by  by &region;
run;


%let chunk_sz=50;

%do k=1 %to %eval(1+(&nb_wgts/&chunk_sz));
	%let start=%eval((&k-1)*&chunk_sz+1);
	%let stop=%eval(%sysfunc(min(&k*&chunk_sz,&nb_wgts)));
  data poids;
    set _rgr_set;
    keep bsw&start-bsw&stop &dependnt &independ &region;
  run;

  %do i=&start %to &stop;
   proc reg data=poids outest=betas(keep=&region &__interc &independ) noprint;
     model &dependnt=&independ;
     weight bsw&i;
&by  by &region;
   run;

   proc transpose data=betas out=betat prefix=best name=beta;
     var &__interc &independ;
&by  by &region;
   run;

   data betat;
     set betat;
     drop _label_;
     rename best1=best&i;
   run;

   %if (&i =1) %then %do;

     data bsbeta;
       set betat;
     run;

   %end;
   %else %do;

     data bsbeta;
       merge bsbeta betat;
&by    by &region;
     run;

   %end;
  %put Step &i Completed;
  %end;
%end;

data est;
  merge origest(rename=(bhat1=param)) bsbeta(rename=(beta=variabls));
  bs_var=((&nb_wgts-1)*(var(of best1-best&nb_wgts)))/&nb_wgts;
  bs_sd=sqrt(bs_var);
  bs_cv=abs(round((bs_sd/param)*100,.01));
  cil95=param-1.96*bs_sd;
  ciu95=param+1.96*bs_sd;
  length dep_var $32;
  dep_var="&dependnt";
  drop best1-best&nb_wgts beta;
  label
  	variabls="Variables"
    param	="Parameter"
	dep_var	="dependnt Variable"
  	bs_var	="Bootstrap Variance"
	bs_sd	="Bootstrap Standard Deviation"
	bs_cv	="Bootstrap Coefficient of Variation"
	cil95	="Lower 95% Confidence Limit"
	ciu95	="Upper 95% Confidence Limit"
  ;
run;

proc append data=est base=bs_reg force; run;

proc datasets library=work; delete _rgr_set; run;

%mend regress;


*********************************************;

%macro logistic(dataset=,dependnt=,independ=,nb_wgts=);
*********************************************;
%local chunk_sz start stop k i;

data _rgr_set;
	set &dataset;
run;

&by proc sort data=_rgr_set; 
&by by &region; 
&by run; 

proc logistic data=_rgr_set outest=orig(keep=&region &__interc &independ) descending noprint;
  model &dependnt=&independ;
  &by by &region;
  weight fwgt;
run;

proc transpose data=orig out=origest prefix=bhat name=beta;
  var &__interc &independ;
  &by by &region;
run;

%let chunk_sz=50;

%do k=1 %to %eval(1+(&nb_wgts/&chunk_sz));
	%let start=%eval((&k-1)*&chunk_sz+1);
	%let stop=%eval(%sysfunc(min(&k*&chunk_sz,&nb_wgts)));

  data poids;
    set _rgr_set;
    keep bsw&start-bsw&stop &dependnt &independ &region;
  run;

  %do i=&start %to &stop;

    proc logistic data=poids outest=betas(keep=&region &__interc &independ) noprint descending;
      model &dependnt=&independ;
      &by by &region;
      weight bsw&i;
    run;

    proc transpose data=betas out=betat prefix=best name=beta;
      var &__interc &independ;
      &by by &region;
    run;

    data betat;
      set betat;
      rename best1=best&i;
    run;

    %if (&i =1) %then %do;

      data bsbeta;
        set betat;
      run;

    %end;
    %else %do;

      data bsbeta;
        merge bsbeta betat;
        &by by &region;
      run;

    %end;
  %put Step &i Completed;
  %end;
%end;

data est;
  merge bsbeta(rename=(beta=variabls)) origest(rename=(bhat1=param));
  bs_var=((&nb_wgts-1)*(var(of best1-best&nb_wgts)))/&nb_wgts;
  bs_sd=sqrt(bs_var);
  bs_cv=abs(round((bs_sd/param)*100,.01));
  wald=(param/bs_sd)*(param/bs_sd);
  pvalue=1-probchi(wald,1);
  lo95=param-1.96*bs_sd;
  hi95=param+1.96*bs_sd;
  odds=exp(param);
  cil95=exp(lo95);
  ciu95=exp(hi95);
  dep_var="&dependnt";
  drop best1-best&nb_wgts beta _label_;
  label
  	wald="Wald Statistic"
  	variabls="Variables"
    param	="Parameter"
	odds	="Odds Ratio"
	dep_var	="dependnt Variable"
  	bs_var	="Bootstrap Variance"
	bs_sd	="Bootstrap Standard Deviation"
	bs_cv	="Bootstrap Coefficient of Variation"
	lo95	="Lower 95% Confidence Limit"
	hi95	="Upper 95% Confidence Limit"
	cil95	="Lower 95% Confidence Limit for Odds Ratio"
	ciu95	="Upper 95% Confidence Limit for Odds Ratio"
  ;
run;

proc append data=est base=bs_reglg;
run;

%mend logistic;

%macro partitio(domains=);
%global region by;
%if &domains=  %then %do;
	%let by=*;
	%let region=;
  %end;
%else %do;
    %let by=;        
	%let region=&domains; 
  %end;
%mend partitio;
%partitio(domains=); *no partition by default;
%macro startchr;
	%global start_tm;
	%let start_tm=%sysfunc(datetime());
%mend startchr;

%macro stop_chr;
	%let elapsed=%sysevalf(%sysfunc(datetime())-&start_tm);
	%let elapsed=%sysevalf(%sysfunc(round(&elapsed,0.001)));
	%let check=%eval(&start_tm<0);
	%if  &check %then
		%do;
			%put Elapsed time: &elapsed  seconds;
			%put Did you start the chronometer?;
		%end;
		%else 
			%put Elapsed time: &elapsed  seconds;
	%let start_tm=-1.0;
%mend stop_chr;

proc printto new log=log; run;
