														  
*                                    AVERTISSEMENT

* Le gouvernement du Canada (Statistique Canada) possde tous les droits de proprit
* intellectuelle (y compris le droit d'auteur) relatifs  ce logiciel.  Sous rserve
* des modalits dcrites ci-dessous, vous obtenez une licence non exclusive et non
* transmissible d'utilisation de ce logiciel.
*
* Ce logiciel vous est fourni  tel quel , et le titulaire ne donne aucune garantie
* expresse ou implicite, ce qui comprend les garanties de qualit marchande et de
* convenance  une fin particulire mais ne se limite pas  ces garanties.  En aucune
* circonstance le titulaire ne sera responsable de dommages indirects, particuliers ou
* conscutifs, ou d'autres dommages semblables.  Ce contrat sera automatiquement rsili,
* sans pravis, si vous omettez de respecter une quelconque des modalits du contrat.;

*                                    WARNING

* The Government of Canada (Statistics Canada) is the owner of all intellectual
* property rights (including copyright) in this software.  Subject to the terms below,
* you are granted a non-exclusive and non-transferable licence to use this software.
*
* This software is provided "as-is", and the owner makes no warranty, either express
* or implied, including but not limited to, warranties of merchantability and fitness
* for any particular purpose.  In no event will the owner be liable for any indirect,
* special, consequential or other similar damages.  This agreement will terminate
* automatically without notice to you if you fail to comply with any term of this
* agreement.;

proc printto new log=dump_log;  run;

%macro chk_vers;
%if (&sysver < 8) %then %do;
	proc printto new log=log; run;
	%put These macros will only work with SAS Version 8.0 or higher.
		It appears that you are running a lower version. You can use
		the bacwards compatible version of these macros - released together
		with this file ;
%end;
%mend check_version;
%chk_vers;


*************************************;

%macro total(dataset=,variable=,nb_poids=);

*************************************;
   proc datasets library=work;
     delete ytot est;
   run;

proc means data=&dataset noprint;
  var fwgt bsw1-bsw&nb_poids;
  weight &variable;
&by  class &region;
  output out=ytot
         sum=Sum ybs1-ybs&nb_poids;
run;

proc sort data=ytot; by descending _type_ &region; run;

data est;
  set ytot;
  length variable $ 32;
  bs_var=((&nb_poids-1)*(var(of ybs1-ybs&nb_poids)))/&nb_poids;
  bs_sd=sqrt(bs_var);
  bs_cv=round((bs_sd/Sum)*100,.01);
  cil95=Sum-1.96*bs_sd;
  ciu95=Sum+1.96*bs_sd;
  variable="&variable";
  drop ybs1-ybs&nb_poids _type_ _freq_;
  label 
  	Sum		="Somme"
  	variable="Variable"
  	bs_var	="Variance Bootstrap"
	bs_sd	="cart Type Bootstrap"
	bs_cv	="Coefficient de Variation Bootstrap"
	cil95	="Limite Inf. Intervalle de Confiance 95%"
	ciu95	="Limite Sup. Intervalle de Confiance 95%"
	;
run;

proc append data=est base=totals force;
run;


%mend total;


*******************************************;

%macro rapport(dataset=,Numrateur=,Dnominateur=,nb_poids=);

*******************************************;
   proc datasets library=work;
     delete ytot xtot est;
   run;

proc means data=&dataset(where=(&Numrateur^=. and &Dnominateur^=.)) noprint;
  var fwgt bsw1-bsw&nb_poids;
  weight &Numrateur;
&by  class &region;
  output out=ytot
         sum=yhat ybs1-ybs&nb_poids;
run;

proc means data=&dataset(where=(&Numrateur^=. and &Dnominateur^=.)) noprint;
  var fwgt bsw1-bsw&nb_poids;
  weight &Dnominateur;
&by  class &region;
  output out=xtot
         sum=xhat xbs1-xbs&nb_poids;
run;

data est;
  merge ytot xtot;
  array ybs{&nb_poids};
  array xbs{&nb_poids};
  array rbs{&nb_poids};
  length Numer $ 32;
  length Denom $ 32;
  ratio=((yhat/xhat)*100);
  do i=1 to &nb_poids;
    rbs{i}=((ybs{i}/xbs{i})*100);
  end;
  bs_var=((&nb_poids-1)*(var(of rbs1-rbs&nb_poids)))/&nb_poids;
  bs_sd=sqrt(bs_var);
  bs_cv=round((bs_sd/ratio)*100,.01);
  cil95=ratio-1.96*bs_sd;
  ciu95=ratio+1.96*bs_sd;
  Numer="&Numrateur";
  Denom="&Dnominateur";
  drop 	ybs1-ybs&nb_poids 
		xbs1-xbs&nb_poids 
		rbs1-rbs&nb_poids 
		xhat yhat i _freq_;
  label
  	Numer	="Numrateur"
	Denom	="Dnominateur"
	ratio	="Rapport - en Pourcentage"
  	bs_var	="Variance Bootstrap"
	bs_sd	="cart Type Bootstrap"
	bs_cv	="Coefficient de Variation Bootstrap"
	cil95	="Limite Inf. Intervalle de Confiance 95%"
	ciu95	="Limite Sup. Intervalle de Confiance 95%"
  ;
run;

proc sort data=est out=est(drop=_type_) ; 
by descending _type_ &region; run;

proc append data=est base=ratios force;
run;


%mend rapport;


********************************************************;

%macro diff_rapports(dataset=,Numrateur1=,Dnominateur1=,
					Numrateur2=,Dnominateur2=,nb_poids=);

********************************************************;

   proc datasets library=work;
     delete ytot xtot yytot xxtot est;
   run;

%local condition;
%let condition= &Numrateur1^=. and &Dnominateur1^=. and &Numrateur2^=. and &Dnominateur2^=.;

proc means data=&dataset(where=(&condition)) noprint;
  var fwgt bsw1-bsw&nb_poids;
  weight &Numrateur1;
&by  class &region;
  output out=ytot
         sum=yhat ybs1-ybs&nb_poids;
run;

proc means data=&dataset(where=(&condition)) noprint;
  var fwgt bsw1-bsw&nb_poids;
  weight &Dnominateur1;
&by  class &region;
  output out=xtot
         sum=xhat xbs1-xbs&nb_poids;
run;

proc means data=&dataset(where=(&condition)) noprint;
  var fwgt bsw1-bsw&nb_poids;
  weight &Numrateur2;
&by  class &region;
  output out=yytot
         sum=yyhat yybs1-yybs&nb_poids;
run;

proc means data=&dataset(where=(&condition)) noprint;
  var fwgt bsw1-bsw&nb_poids;
  weight &Dnominateur2;
&by  class &region;
  output out=xxtot
         sum=xxhat xxbs1-xxbs&nb_poids;
run;

data est;
  merge ytot xtot yytot xxtot;
  array ybs{&nb_poids};
  array xbs{&nb_poids};
  array yybs{&nb_poids};
  array xxbs{&nb_poids};
  array drbs{&nb_poids};
  length Numer1 $ 32;
  length Denom1 $ 32;
  length Numer2 $ 32;
  length Denom2 $ 32;
  diffrat=(((yhat/xhat)-(yyhat/xxhat))*100);
  do i=1 to &nb_poids;
    drbs{i}=(((ybs{i}/xbs{i})-(yybs{i}/xxbs{i}))*100);
  end;
  bs_var=(((&nb_poids-1)*(var(of drbs1-drbs&nb_poids)))/&nb_poids);
  bs_sd=sqrt(bs_var);
  bs_cv=abs(round((bs_sd/diffrat)*100,.01));
  cil95=diffrat-1.96*bs_sd;
  ciu95=diffrat+1.96*bs_sd;
  Numer1="&Numrateur1";
  Denom1="&Dnominateur1";
  Numer2="&Numrateur2";
  Denom2="&Dnominateur2";
  drop 	ybs1-ybs&nb_poids 
		xbs1-xbs&nb_poids 
		yybs1-yybs&nb_poids 
		xxbs1-xxbs&nb_poids 
		drbs1-drbs&nb_poids
       	xhat yhat xxhat yyhat i _freq_;
  label
  	Numer1	="Numrateur - Premier Rapport"
	Denom1	="Dnominateur - Premier Rapport"
  	Numer2	="Numrateur - Deuxime Rapport"
	Denom2	="Dnominateur - Deuxime Rapport"
	diffrat	="Diffrence des Pourcentages"
  	bs_var	="Variance Bootstrap"
	bs_sd	="cart Type Bootstrap"
	bs_cv	="Coefficient de Variation Bootstrap"
	cil95	="Limite Inf. Intervalle de Confiance 95%"
	ciu95	="Limite Sup. Intervalle de Confiance 95%"
  ;
run;

proc sort data=est out=est(drop=_type_); by descending _type_ &region; run;

proc append data=est base=diffrat force;
run;

%mend diff_rapports;


*********************************************;

%macro regression(dataset=,dependent=,independent=,nb_poids=);

*********************************************;
%local chunk_size start stop k i;


data _rgr_set;
	set &dataset;
run;

&by proc sort data=_rgr_set; 
&by by &region; 
&by run; 

   proc datasets library=work;
     delete betas betat bsbeta origest est;
   run;

proc reg data=_rgr_set outest=orig(keep=&region intercept &independent) noprint;
  model &dependent=&independent;
  weight fwgt;
&by  by &region;
run;

proc transpose data=orig out=origest(drop=_label_) prefix=bhat name=beta;
  var intercept &independent;
&by  by &region;
run;


%let chunk_size=50;

%do k=1 %to %eval(1+(&nb_poids/&chunk_size));
	%let start=%eval((&k-1)*&chunk_size+1);
	%let stop=%eval(%sysfunc(min(&k*&chunk_size,&nb_poids)));
  data poids;
    set _rgr_set;
    keep bsw&start-bsw&stop &dependent &independent &region;
  run;

  %do i=&start %to &stop;
   proc reg data=poids outest=betas(keep=&region intercept &independent) noprint;
     model &dependent=&independent;
     weight bsw&i;
&by  by &region;
   run;

   proc transpose data=betas out=betat prefix=best name=beta;
     var intercept &independent;
&by  by &region;
   run;

   data betat;
     set betat;
     drop _label_;
     rename best1=best&i;
   run;

   %if (&i =1) %then %do;

     data bsbeta;
       set betat;
     run;

   %end;
   %else %do;

     data bsbeta;
       merge bsbeta betat;
&by    by &region;
     run;

   %end;
  %put Pas &i Complet;
  %end;
%end;

data est;
  merge origest(rename=(bhat1=param)) bsbeta(rename=(beta=variabls));
  bs_var=((&nb_poids-1)*(var(of best1-best&nb_poids)))/&nb_poids;
  bs_sd=sqrt(bs_var);
  bs_cv=abs(round((bs_sd/param)*100,.01));
  cil95=param-1.96*bs_sd;
  ciu95=param+1.96*bs_sd;
  length dep_var $32;
  dep_var="&dependent";
  drop best1-best&nb_poids beta;
  label
  	variabls="Variables"
    param	="Paramtres"
	dep_var	="Variable Dpendente "
  	bs_var	="Variance Bootstrap"
	bs_sd	="cart Type Bootstrap"
	bs_cv	="Coefficient de Variation Bootstrap"
	cil95	="Limite Inf. Intervalle de Confiance 95%"
	ciu95	="Limite Sup. Intervalle de Confiance 95%"
  ;
run;

proc append data=est base=bs_reg force; run;

proc datasets library=work; delete _rgr_set; run;

%mend regression;


*********************************************;

%macro reg_logistique(dataset=,dependent=,independent=,nb_poids=);
*********************************************;
%local chunk_size start stop k i;

data _rgr_set;
	set &dataset;
run;

&by proc sort data=_rgr_set; 
&by by &region; 
&by run; 

proc logistic data=_rgr_set outest=orig(keep=&region intercept &independent) descending noprint;
  model &dependent=&independent;
  &by by &region;
  weight fwgt;
run;

proc transpose data=orig out=origest prefix=bhat name=beta;
  var intercept &independent;
  &by by &region;
run;

%let chunk_size=50;

%do k=1 %to %eval(1+(&nb_poids/&chunk_size));
	%let start=%eval((&k-1)*&chunk_size+1);
	%let stop=%eval(%sysfunc(min(&k*&chunk_size,&nb_poids)));

  data poids;
    set _rgr_set;
    keep bsw&start-bsw&stop &dependent &independent &region;
  run;

  %do i=&start %to &stop;

    proc logistic data=poids outest=betas(keep=&region intercept &independent) noprint descending;
      model &dependent=&independent;
      &by by &region;
      weight bsw&i;
    run;

    proc transpose data=betas out=betat prefix=best name=beta;
      var intercept &independent;
      &by by &region;
    run;

    data betat;
      set betat;
      rename best1=best&i;
    run;

    %if (&i =1) %then %do;

      data bsbeta;
        set betat;
      run;

    %end;
    %else %do;

      data bsbeta;
        merge bsbeta betat;
        &by by &region;
      run;

    %end;
  %put Pas &i Complet;
  %end;
%end;

data est;
  merge bsbeta(rename=(beta=variabls)) origest(rename=(bhat1=param));
  bs_var=((&nb_poids-1)*(var(of best1-best&nb_poids)))/&nb_poids;
  bs_sd=sqrt(bs_var);
  bs_cv=abs(round((bs_sd/param)*100,.01));
  wald=(param/bs_sd)*(param/bs_sd);
  pvalue=1-probchi(wald,1);
  lo95=param-1.96*bs_sd;
  hi95=param+1.96*bs_sd;
  odds=exp(param);
  cil95=exp(lo95);
  ciu95=exp(hi95);
  dep_var="&dependent";
  drop best1-best&nb_poids beta _label_;
  label
  	wald="Statistique de Wald"
  	variabls="Variables"
    param	="Paramtres"
	odds	="Rapport des Cotes"
	dep_var	="Variable Dpendente"
  	bs_var	="Variance Bootstrap"
	bs_sd	="cart Type Bootstrap"
	bs_cv	="Coefficient de Variation Bootstrap"
	lo95	="Limite Inf. Intervalle de Confiance 95%"
	hi95	="Limite Sup. Intervalle de Confiance 95%"
	cil95	="Limite Inf. Intervalle de Confiance 95% pour Rapport des Cotes"
	ciu95	="Limite Sup. Intervalle de Confiance 95% pour Rapport des Cotes"
  ;
run;

proc append data=est base=bs_reglg;
run;

%mend reg_logistique;

%macro partition(domains=);
%global region by;
%if &domains=  %then %do;
	%let by=*;
	%let region=;
  %end;
%else %do;
    %let by=;        
	%let region=&domains; 
  %end;
%mend partition;
%partition(domains=); *pas de partition;
%macro start_chrono;
	%global start_time;
	%let start_time=%sysfunc(datetime());
%mend start_chron;

%macro stop_chrono;
	%let elapsed=%sysevalf(%sysfunc(datetime())-&start_time);
	%let elapsed=%sysevalf(%sysfunc(round(&elapsed,0.001)));
	%let check=%eval(&start_time<0.001);
	%if  &check %then
		%do;
			%put Intrval de temps: &elapsed  secondes;
			%put Avez vous innitialis le chrono?;
		%end;
		%else 
			%put Intrval de temps: &elapsed  secondes;
	%let start_time=0.0;
%mend stop_chrono;

proc printto new log=log; run;
